/*
* main.c - contains program main
*
* Copyright (c) 2012-2015 emotas embedded communication GmbH
*-------------------------------------------------------------------
* $Id: main.c 32405 2020-06-15 09:05:46Z hil $
*
*
*-------------------------------------------------------------------
*
*
*/

/********************************************************************/
/**
* \file
* \brief main routine of the XMC4000 CANopen slave example
*
*/

/* header of standard C - libraries
---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* header of project specific types
---------------------------------------------------------------------------*/
#include <gen_define.h>
#include <co_canopen.h>

/* dave XMC Lib  interface definition */
#include <xmc_gpio.h>

/* constant definitions
---------------------------------------------------------------------------*/

/* local defined data types
---------------------------------------------------------------------------*/

/* list of external used functions, if not in headers
--------------------------------------------------------------------------*/

/* list of global defined functions
---------------------------------------------------------------------------*/

/* list of local defined functions
---------------------------------------------------------------------------*/
static RET_T nmtInd(BOOL_T	execute, CO_NMT_STATE_T newState);
static void hbState(UNSIGNED8	nodeId, CO_ERRCTRL_T state,
		CO_NMT_STATE_T	nmtState);
static RET_T sdoServerReadInd(BOOL_T execute, UNSIGNED8	sdoNr, UNSIGNED16 index,
		UNSIGNED8	subIndex);
static RET_T sdoServerCheckWriteInd(BOOL_T execute, UNSIGNED8 node,
		UNSIGNED16 index, UNSIGNED8	subIndex, const UNSIGNED8 *pData);
static RET_T sdoServerWriteInd(BOOL_T execute, UNSIGNED8 sdoNr,
		UNSIGNED16 index, UNSIGNED8	subIndex);
static void pdoInd(UNSIGNED16);
static void pdoRecEvent(UNSIGNED16);
static void canInd(CO_CAN_STATE_T);
static void commInd(CO_COMM_STATE_EVENT_T);
static void ledGreenInd(BOOL_T);
static void ledRedInd(BOOL_T);
static void errorHandler(int errorCode);
static void timerCallback(void* pVoid);

/* external variables
---------------------------------------------------------------------------*/

/* global variables
---------------------------------------------------------------------------*/

/* local defined variables
---------------------------------------------------------------------------*/
CO_TIMER_T timer1;

/***************************************************************************/
/**
* \brief main entry
*
* \param
*	nothing
* \results
*	nothing
*/
int main(
	void
  )
{
UNSIGNED8	emcyData[5] = { 1u, 2u, 3u, 4u, 5u };

	/* Initialize all hardware needed by the CANopen stack:
	 * CAN controller clock,
	 * CAN controller RX/TX pins
	 * used LEDs
	 */
	codrvHardwareInit();

	if (codrvCanInit(250u) != RET_OK)  {
		errorHandler(1);
	}

	if (codrvTimerSetup(CO_TIMER_INTERVAL) != RET_OK)  {
		errorHandler(2);
	}

	if (coCanOpenStackInit(NULL) != RET_OK)  {
		printf("error init library\n");
		errorHandler(1);
	}

	/* CANopen Initialization */

	/* register indication functions */
	if (coEventRegister_NMT(nmtInd) != RET_OK)  {
		errorHandler(3);
	}
	if (coEventRegister_ERRCTRL(hbState) != RET_OK)  {
		errorHandler(4);
	}
	if (coEventRegister_SDO_SERVER_READ(sdoServerReadInd) != RET_OK)  {
		errorHandler(5);
	}
	if (coEventRegister_SDO_SERVER_CHECK_WRITE(sdoServerCheckWriteInd) != RET_OK)  {
		errorHandler(6);
	}
	if (coEventRegister_SDO_SERVER_WRITE(sdoServerWriteInd) != RET_OK)  {
		errorHandler(7);
	}
	if (coEventRegister_PDO(pdoInd) != RET_OK)  {
		errorHandler(8);
	}
	if (coEventRegister_PDO_REC_EVENT(pdoRecEvent) != RET_OK)  {
		errorHandler(9);
	}
	if (coEventRegister_LED_GREEN(ledGreenInd) != RET_OK)  {
		errorHandler(10);
	}
	if (coEventRegister_LED_RED(ledRedInd) != RET_OK)  {
		errorHandler(11);
	}
	if (coEventRegister_CAN_STATE(canInd) != RET_OK)  {
		errorHandler(12);
	}
	if (coEventRegister_COMM_EVENT(commInd) != RET_OK)  {
		errorHandler(13);
	}

	/* enable CAN communication */
	if (codrvCanEnable() != RET_OK)  {
		errorHandler(14);
	}

	/* send emcy message */
	if (coEmcyWriteReq(0x1234, &emcyData[0]) != RET_OK)  {
		errorHandler(15);
	}

	/* create cyclic timer */
	if (coTimerStart(&timer1, (1000ul * 1000ul), timerCallback, NULL, CO_TIMER_ATTR_ROUNDUP_CYCLIC) != RET_OK)  {
		errorHandler(16);
	}


	/* CANopen main loop */
	while (1)  {
		while (coCommTask() == CO_TRUE)  {
			/* more CAN messages to process
			 * coCommTask() is called again */
		}
	}
}


/*********************************************************************/
static void timerCallback(void* pVoid)
{
UNSIGNED8 val_2000_1;

	/* read object dictionary entry 0x2000:1 */
	if (coOdGetObj_u8(0x2000, 1u, &val_2000_1) == RET_OK)  {
		/* increase value */
		val_2000_1++;

		/* write increased value back to the object dictionary */
		if (coOdPutObj_u8(0x2000, 1u, val_2000_1) != RET_OK)  {
			printf("something went wrong\n");
		}
	}
}

/*********************************************************************/
static RET_T nmtInd(
		BOOL_T	execute,
		CO_NMT_STATE_T	newState
	)
{
	printf("nmtInd: New Nmt state %d - execute %d\n", newState, execute);

	return(RET_OK);
}


/*********************************************************************/
static void pdoInd(
		UNSIGNED16	pdoNr
	)
{
	printf("pdoInd: pdo %d received\n", pdoNr);
}


/*********************************************************************/
static void pdoRecEvent(
		UNSIGNED16	pdoNr
	)
{
	printf("pdoRecEvent: pdo %d time out\n", pdoNr);
}


/*********************************************************************/
static void hbState(
		UNSIGNED8	nodeId,
		CO_ERRCTRL_T state,
		CO_NMT_STATE_T	nmtState
	)
{
	printf("hbInd: HB Event %d node %d nmtState: %d\n", state, nodeId, nmtState);

    return;
}


/*********************************************************************/
static RET_T sdoServerReadInd(
		BOOL_T		execute,
		UNSIGNED8	sdoNr,
		UNSIGNED16	index,
		UNSIGNED8	subIndex
	)
{
	printf("sdo server read ind: exec: %d, sdoNr %d, index %x:%d\n",
		execute, sdoNr, index, subIndex);
	/* if an read request should not accepted by the application
	 * simply return with an error code
	 * return(RET_INVALID_PARAMETER);
	 */
	return(RET_OK);
}


/*********************************************************************/
static RET_T sdoServerCheckWriteInd(
		BOOL_T		execute,
		UNSIGNED8	sdoNr,
		UNSIGNED16	index,
		UNSIGNED8	subIndex,
		const UNSIGNED8	*pData
	)
{
	printf("sdo server check write ind: exec: %d, sdoNr %d, index %x:%d\n",
		execute, sdoNr, index, subIndex);
	/* if an write request should not accepted by the application
	 * simply return with an error code
	 * return(RET_INVALID_PARAMETER);
	 */
	return(RET_OK);
}


/*********************************************************************/
static RET_T sdoServerWriteInd(
		BOOL_T		execute,
		UNSIGNED8	sdoNr,
		UNSIGNED16	index,
		UNSIGNED8	subIndex
	)
{
	printf("sdo server write ind: exec: %d, sdoNr %d, index %x:%d\n",
		execute, sdoNr, index, subIndex);

	return(RET_OK);
}


/*********************************************************************/
static void canInd(
	CO_CAN_STATE_T	canState
	)
{
	switch (canState)  {
		case CO_CAN_STATE_BUS_OFF:
			printf("CAN: Bus Off\n");
			break;
		case CO_CAN_STATE_BUS_ON:
			printf("CAN: Bus On\n");
			break;
		case CO_CAN_STATE_PASSIVE:
			printf("CAN: Passive\n");
			break;
		default:
			break;
	}
}


/*********************************************************************/
static void commInd(
		CO_COMM_STATE_EVENT_T	commEvent
	)
{
	switch (commEvent)  {
		case CO_COMM_STATE_EVENT_CAN_OVERRUN:
			printf("COMM-Event: CAN Overrun\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
			printf("COMM-Event: Rec Queue Full\n");
			break;
		case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
			printf("COMM-Event: Rec Queue Overflow\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
			printf("COMM-Event: Tr Queue Full\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
			printf("COMM-Event: Tr Queue Overflow\n");
			break;
		case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
			printf("COMM-Event: Tr Queue Empty\n");
			break;
		default:
			printf("COMM-Event: %d\n", commEvent);
			break;
	}

}

/*********************************************************************/
static void ledGreenInd(
    BOOL_T  on
)
{
#ifdef CANOPEN_LED
	/* hardware control of the CANopen Green LED
	 * Low level on Pin switches LED on.
	 */
	if (on == CO_TRUE) {
		XMC_GPIO_SetOutputLow(CANOPEN_LED);
	} else {
		XMC_GPIO_SetOutputHigh(CANOPEN_LED);
	}
#endif /* CANOPEN_LED */
}


/*********************************************************************/
/* No additional LED is available on the CPU Board XMC4500 General Purpose */
static void ledRedInd(
    BOOL_T  on
)
{
#ifdef ERROR_LED
	/* hardware control for the CANopen Error LED
	 */
	if (on == CO_TRUE) {
		XMC_GPIO_SetOutputLow(ERROR_LED);
	} else {
		XMC_GPIO_SetOutputLow(ERROR_LED);
	}
#endif /* ERROR_LED */
}


/*********************************************************************/
static void errorHandler(
		int errorCode
	)
{
	while(1) { (void)errorCode; };
}
